@@ -2,7 +2,7 @@
  * include/version.h
  * Version number header file
  *
- * Copyright (c) 2017-2021 Roderick W. Smith
+ * Copyright (c) 2017-2024 Roderick W. Smith
  *
  * Distributed under the terms of the GNU General Public License (GPL)
  * version 3 (GPLv3), a copy of which must be distributed with this
@@ -10,5 +10,9 @@
  *
  */
 
-#define REFIND_VERSION L"0.13.2"
+#ifndef __VERSION_H
+#define __VERSION_H
 
+#define REFIND_VERSION L"0.14.2"
+
+#endif
