@@ -25,6 +25,7 @@
 
 #include "global.h"
 #include "../refind/screen.h"
+#include "../refind/lib.h"
 #include "lodepng.h"
 
 // EFI's equivalent of realloc requires the original buffer's size as an
@@ -51,7 +52,7 @@ void* lodepng_malloc(size_t size) {
 void lodepng_free (void *ptr) {
    if (ptr) {
       ptr = (void *) (((size_t *) ptr) - 1);
-      FreePool(ptr);
+      MyFreePool(ptr);
    }
 } // void lodepng_free()
 
@@ -69,7 +70,7 @@ void* lodepng_realloc(void *ptr, size_t new_size) {
    new_pool = lodepng_malloc(new_size);
    if (new_pool && ptr) {
       old_size = report_size(ptr);
-      CopyMem(new_pool, ptr, (old_size < new_size) ? old_size : new_size);
+      MyCopyMem(new_pool, ptr, (old_size < new_size) ? old_size : new_size);
    }
    return new_pool;
 } // lodepng_realloc()
@@ -91,7 +92,7 @@ VOID *MyMemSet(VOID *s, int c, size_t n) {
 }
 
 VOID *MyMemCpy(void *__restrict __dest, const void *__restrict __src, size_t __n) {
-    CopyMem(__dest, __src, __n);
+    MyCopyMem(__dest, __src, __n);
     return __dest;
 }
 
