@@ -35,7 +35,7 @@
  */
 
 /*
- * Modifications copyright (c) 2012-2021 Roderick W. Smith
+ * Modifications copyright (c) 2012-2024 Roderick W. Smith
  * 
  * Modifications distributed under the terms of the GNU General Public
  * License (GPL) version 3 (GPLv3) or (at your option) any later version.
@@ -81,9 +81,6 @@
 #define GetTime ST->RuntimeServices->GetTime
 #define LAST_MINUTE 1439 /* Last minute of a day */
 
-// extern REFIT_MENU_ENTRY MenuEntryReturn;
-//static REFIT_MENU_ENTRY MenuEntryReturn   = { L"Return to Main Menu", TAG_RETURN, 0, 0, 0, NULL, NULL, NULL };
-
 //
 // read a file into a buffer
 //
@@ -114,7 +111,7 @@ EFI_STATUS ReadFile(IN EFI_FILE_HANDLE BaseDir, IN CHAR16 *FileName, IN OUT REFI
         return EFI_LOAD_ERROR;
     }
     ReadSize = FileInfo->FileSize;
-    FreePool(FileInfo);
+    MyFreePool(FileInfo);
 
     File->BufferSize = (UINTN)ReadSize;
     File->Buffer = AllocatePool(File->BufferSize);
@@ -260,7 +257,7 @@ static BOOLEAN KeepReading(IN OUT CHAR16 *p, IN OUT BOOLEAN *IsQuoted) {
             Temp = StrDuplicate(&p[1]);
             if (Temp != NULL) {
                 StrCpy(p, Temp);
-                FreePool(Temp);
+                MyFreePool(Temp);
             }
             MoreToRead = TRUE;
         } else {
@@ -316,7 +313,7 @@ UINTN ReadTokenLine(IN REFIT_FILE *File, OUT CHAR16 ***TokenList)
             AddListElement((VOID ***)TokenList, &TokenCount, (VOID *)StrDuplicate(Token));
         }
 
-        FreePool(Line);
+        MyFreePool(Line);
     }
     return (TokenCount);
 } /* ReadTokenLine() */
@@ -360,7 +357,7 @@ static VOID HandleStrings(IN CHAR16 **TokenList, IN UINTN TokenCount, OUT CHAR16
     }
 
     if ((*Target != NULL) && !AddMode) {
-        FreePool(*Target);
+        MyFreePool(*Target);
         *Target = NULL;
     } // if
     for (i = 1; i < TokenCount; i++) {
@@ -494,6 +491,24 @@ static LOADER_ENTRY * AddPreparedLoaderEntry(LOADER_ENTRY *Entry) {
     return(Entry);
 } // LOADER_ENTRY * AddPreparedLoaderEntry()
 
+// Sets GlobalConfig.LinuxMatchPatterns based on the input comma-delimited set
+// of prefixes. An asterisk ("*") is added to each of the input prefixes and
+// GlobalConfig.LinuxMatchPatterns is set to the resulting comma-delimited
+// string.
+static VOID SetLinuxMatchPatterns(CHAR16 *Prefixes) {
+    CHAR16 *Pattern, *PatternSet = NULL;
+    UINTN i = 0;
+
+    while ((Pattern = FindCommaDelimited(Prefixes, i++)) != NULL) {
+        MergeStrings(&Pattern, L"*", 0);
+        MergeStrings(&PatternSet, Pattern, L',');
+        MyFreePool(Pattern);
+    }
+    if (GlobalConfig.LinuxMatchPatterns)
+        MyFreePool(GlobalConfig.LinuxMatchPatterns);
+    GlobalConfig.LinuxMatchPatterns = PatternSet;
+} // VOID SetLinuxMatchPatterns()
+
 // read config file
 VOID ReadConfig(CHAR16 *FileName)
 {
@@ -512,7 +527,9 @@ VOID ReadConfig(CHAR16 *FileName)
         if (SelfVolume)
             TempStr = GuidAsString(&(SelfVolume->PartGuid));
         MergeStrings(&TempStr, SelfDirPath, L':');
-        MergeStrings(&TempStr, MEMTEST_LOCATIONS, L',');
+        if (GlobalConfig.ToolLocations == NULL)
+            GlobalConfig.ToolLocations = StrDuplicate(TOOL_LOCATIONS);
+        MergeStrings(&TempStr, TOOL_LOCATIONS, L',');
         GlobalConfig.DontScanDirs = TempStr;
         MyFreePool(GlobalConfig.DontScanFiles);
         GlobalConfig.DontScanFiles = StrDuplicate(DONT_SCAN_FILES);
@@ -522,12 +539,15 @@ VOID ReadConfig(CHAR16 *FileName)
         GlobalConfig.DontScanFirmware = NULL;
         MergeStrings(&(GlobalConfig.DontScanFiles), MOK_NAMES, L',');
         MergeStrings(&(GlobalConfig.DontScanFiles), FWUPDATE_NAMES, L',');
+        MergeStrings(&(GlobalConfig.DontScanFiles), MEMTEST_NAMES, L',');
+        MergeStrings(&(GlobalConfig.DontScanFiles), SHELL_NAMES, L',');
         MyFreePool(GlobalConfig.DontScanVolumes);
         GlobalConfig.DontScanVolumes = StrDuplicate(DONT_SCAN_VOLUMES);
         GlobalConfig.WindowsRecoveryFiles = StrDuplicate(WINDOWS_RECOVERY_FILES);
         GlobalConfig.MacOSRecoveryFiles = StrDuplicate(MACOS_RECOVERY_FILES);
         MyFreePool(GlobalConfig.DefaultSelection);
         GlobalConfig.DefaultSelection = StrDuplicate(L"+");
+        GlobalConfig.LinuxPrefixes = StrDuplicate(LINUX_PREFIXES);
     } // if
 
     if (!FileExists(SelfDir, FileName)) {
@@ -593,6 +613,9 @@ VOID ReadConfig(CHAR16 *FileName)
                     GlobalConfig.ScanFor[i] = ' ';
             }
 
+        } else if (MyStriCmp(TokenList[0], L"follow_symlinks")) {
+            GlobalConfig.FollowSymlinks = HandleBoolean(TokenList, TokenCount);
+
         } else if (MyStriCmp(TokenList[0], L"use_nvram")) {
             GlobalConfig.UseNvram = HandleBoolean(TokenList, TokenCount);
 
@@ -630,6 +653,9 @@ VOID ReadConfig(CHAR16 *FileName)
                    MyStriCmp(TokenList[0], L"dont_scan_firmware")) {
             HandleStrings(TokenList, TokenCount, &(GlobalConfig.DontScanFirmware));
 
+        } else if (MyStriCmp(TokenList[0], L"also_scan_tool_dirs")) {
+            HandleStrings(TokenList, TokenCount, &(GlobalConfig.ExtraToolLocations));
+
         } else if (MyStriCmp(TokenList[0], L"don't_scan_tools") ||
                    MyStriCmp(TokenList[0], L"dont_scan_tools")) {
             HandleStrings(TokenList, TokenCount, &(GlobalConfig.DontScanTools));
@@ -687,6 +713,9 @@ VOID ReadConfig(CHAR16 *FileName)
                 }
             } // showtools options
 
+        } else if (MyStriCmp(TokenList[0], L"support_gzipped_loaders")) {
+           GlobalConfig.GzippedLoaders = HandleBoolean(TokenList, TokenCount);
+
         } else if (MyStriCmp(TokenList[0], L"banner")) {
             HandleString(TokenList, TokenCount, &(GlobalConfig.BannerFileName));
 
@@ -780,6 +809,9 @@ VOID ReadConfig(CHAR16 *FileName)
         } else if (MyStriCmp(TokenList[0], L"fold_linux_kernels")) {
             GlobalConfig.FoldLinuxKernels = HandleBoolean(TokenList, TokenCount);
 
+        } else if (MyStriCmp(TokenList[0], L"linux_prefixes")) {
+            HandleStrings(TokenList, TokenCount, &(GlobalConfig.LinuxPrefixes));
+
         } else if (MyStriCmp(TokenList[0], L"extra_kernel_version_strings")) {
             HandleStrings(TokenList, TokenCount, &(GlobalConfig.ExtraKernelVersionStrings));
 
@@ -831,6 +863,8 @@ VOID ReadConfig(CHAR16 *FileName)
         MergeStrings(&(GlobalConfig.DontScanFiles), GlobalConfig.WindowsRecoveryFiles, L',');
     MyFreePool(File.Buffer);
 
+    SetLinuxMatchPatterns(GlobalConfig.LinuxPrefixes);
+
     if (!FileExists(SelfDir, L"icons") && !FileExists(SelfDir, GlobalConfig.IconsDir)) {
         Print(L"Icons directory doesn't exist; setting textonly = TRUE!\n");
         GlobalConfig.TextOnly = TRUE;
@@ -1081,9 +1115,9 @@ static REFIT_FILE * GenerateOptionsFromEtcFstab(REFIT_VOLUME *Volume) {
         Status = ReadFile(Volume->RootDir, L"\\etc\\fstab", Fstab, &i);
         if (CheckError(Status, L"while reading /etc/fstab")) {
             if (Options != NULL)
-                FreePool(Options);
+                MyFreePool(Options);
             if (Fstab != NULL)
-                FreePool(Fstab);
+                MyFreePool(Fstab);
             Options = NULL;
             Fstab = NULL;
         } else { // File read; locate root fs and create entries
@@ -1200,8 +1234,7 @@ REFIT_FILE * ReadLinuxOptionsFile(IN CHAR16 *LoaderPath, IN REFIT_VOLUME *Volume
                 File = AllocateZeroPool(sizeof(REFIT_FILE));
                 Status = ReadFile(Volume->RootDir, FullFilename, File, &size);
                 if (CheckError(Status, L"while loading the Linux options file")) {
-                    if (File != NULL)
-                        FreePool(File);
+                    MyFreePool(File);
                     File = NULL;
                 } else {
                     GoOn = FALSE;
@@ -1238,8 +1271,7 @@ CHAR16 * GetFirstOptionsFromFile(IN CHAR16 *LoaderPath, IN REFIT_VOLUME *Volume)
         if (TokenCount > 1)
             Options = StrDuplicate(TokenList[1]);
         FreeTokenLine(&TokenList, &TokenCount);
-        FreePool(File);
+        MyFreePool(File);
     } // if
     return Options;
 } // static CHAR16 * GetOptionsFile()
-
