@@ -34,7 +34,7 @@
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 /*
- * Modifications copyright (c) 2012-2021 Roderick W. Smith
+ * Modifications copyright (c) 2012-2024 Roderick W. Smith
  *
  * Modifications distributed under the terms of the GNU General Public
  * License (GPL) version 3 (GPLv3), or (at your option) any later version.
@@ -102,6 +102,12 @@ REFIT_CONFIG GlobalConfig = { /* TextOnly = */ FALSE,
                               /* ShutdownAfterTimeout = */ FALSE,
                               /* Install = */ FALSE,
                               /* WriteSystemdVars = */ FALSE,
+                              /* FollowSymlinks = */ FALSE,
+#ifdef EFIAARCH64
+                              /* GzippedLoaders = */ TRUE,
+#else
+                              /* GzippedLoaders = */ FALSE,
+#endif
                               /* RequestedScreenWidth = */ 0,
                               /* RequestedScreenHeight = */ 0,
                               /* BannerBottomEdge = */ 0,
@@ -129,6 +135,8 @@ REFIT_CONFIG GlobalConfig = { /* TextOnly = */ FALSE,
                               /* *SelectionBigFileName = */ NULL,
                               /* *DefaultSelection = */ NULL,
                               /* *AlsoScan = */ NULL,
+                              /* ToolLocations = */ NULL,
+                              /* *ExtraToolLocations = */ NULL,
                               /* *DontScanVolumes = */ NULL,
                               /* *DontScanDirs = */ NULL,
                               /* *DontScanFiles = */ NULL,
@@ -138,6 +146,8 @@ REFIT_CONFIG GlobalConfig = { /* TextOnly = */ FALSE,
                               /* *MacOSRecoveryFiles = */ NULL,
                               /* *DriverDirs = */ NULL,
                               /* *IconsDir = */ NULL,
+                              /* *LinuxPrefixes = */ NULL,
+                              /* *LinuxMatchPatterns = */ NULL,
                               /* *ExtraKernelVersionStrings = */ NULL,
                               /* *SpoofOSXVersion = */ NULL,
                               /* CsrValues = */ NULL,
@@ -146,6 +156,8 @@ REFIT_CONFIG GlobalConfig = { /* TextOnly = */ FALSE,
                                                   TAG_FWUPDATE_TOOL, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
                             };
 
+CHAR16 *gHiddenTools = NULL;
+
 EFI_GUID RefindGuid = REFIND_GUID_VALUE;
 
 //
@@ -164,7 +176,7 @@ VOID AboutrEFInd(VOID)
         AddMenuInfoLine(&AboutMenu, PoolPrint(L"rEFInd Version %s", REFIND_VERSION));
         AddMenuInfoLine(&AboutMenu, L"");
         AddMenuInfoLine(&AboutMenu, L"Copyright (c) 2006-2010 Christoph Pfisterer");
-        AddMenuInfoLine(&AboutMenu, L"Copyright (c) 2012-2021 Roderick W. Smith");
+        AddMenuInfoLine(&AboutMenu, L"Copyright (c) 2012-2024 Roderick W. Smith");
         AddMenuInfoLine(&AboutMenu, L"Portions Copyright (c) Intel Corporation and others");
         AddMenuInfoLine(&AboutMenu, L"Distributed under the terms of the GNU GPLv3 license");
         AddMenuInfoLine(&AboutMenu, L"");
@@ -176,14 +188,17 @@ VOID AboutrEFInd(VOID)
                                               ST->FirmwareRevision & ((1 << 16) - 1)));
         AddMenuInfoLine(&AboutMenu, PoolPrint(L" EFI Revision %d.%02d", ST->Hdr.Revision >> 16, ST->Hdr.Revision & ((1 << 16) - 1)));
 #if defined(EFI32)
-        AddMenuInfoLine(&AboutMenu, PoolPrint(L" Platform: x86 (32 bit); Secure Boot %s",
-                                              secure_mode() ? L"active" : L"inactive"));
+        AddMenuInfoLine(&AboutMenu, PoolPrint(L" Platform: x86 (32 bit); Secure Boot %s %s",
+                                              secure_mode() ? L"active" : L"inactive",
+                                              ShimLoaded() ? L"(via Shim)" : L"(direct)"));
 #elif defined(EFIX64)
-        AddMenuInfoLine(&AboutMenu, PoolPrint(L" Platform: x86_64 (64 bit); Secure Boot %s",
-                                              secure_mode() ? L"active" : L"inactive"));
+        AddMenuInfoLine(&AboutMenu, PoolPrint(L" Platform: x86_64 (64 bit); Secure Boot %s %s",
+                                              secure_mode() ? L"active" : L"inactive",
+                                              ShimLoaded() ? L"(via Shim)" : L"(direct)"));
 #elif defined(EFIAARCH64)
-        AddMenuInfoLine(&AboutMenu, PoolPrint(L" Platform: ARM (64 bit); Secure Boot %s",
-                                              secure_mode() ? L"active" : L"inactive"));
+        AddMenuInfoLine(&AboutMenu, PoolPrint(L" Platform: ARM (64 bit); Secure Boot %s %s",
+                                              secure_mode() ? L"active" : L"inactive",
+                                              ShimLoaded() ? L"(via Shim)" : L"(direct)"));
 #else
         AddMenuInfoLine(&AboutMenu, L" Platform: unknown");
 #endif
@@ -241,12 +256,19 @@ VOID RescanAll(BOOLEAN DisplayMessage, BOOLEAN Reconnect) {
 
 // Minimal initialization function
 static VOID InitializeLib(IN EFI_HANDLE ImageHandle, IN EFI_SYSTEM_TABLE *SystemTable) {
+    EFI_STATUS Status;
+
     gST            = SystemTable;
     //    gImageHandle   = ImageHandle;
     gBS            = SystemTable->BootServices;
     //    gRS            = SystemTable->RuntimeServices;
     gRT = SystemTable->RuntimeServices; // Some BDS functions need gRT to be set
-    EfiGetSystemConfigurationTable (&gEfiDxeServicesTableGuid, (VOID **) &gDS);
+    Status = EfiGetSystemConfigurationTable (&gEfiDxeServicesTableGuid, (VOID **) &gDS);
+    if (EFI_ERROR(Status)) {
+        // Be sure that gDS isn't pointing to some random place; check it
+        // before each use....
+        gDS = 0;
+    }
 }
 
 #endif
@@ -433,6 +455,12 @@ VOID LogBasicInfo(VOID) {
     LOG(1, LOG_LINE_NORMAL, L"System does%s support GOP graphics mode",
         EFI_ERROR(Status) ? L" not" : L"");
 
+#ifdef __MAKEWITH_TIANO
+    if (gDS == 0) {
+        LOG(1, LOG_LINE_NORMAL, L"WARNING: EfiGetSystemConfigurationTable() returned error status %lu!", Status);
+        LOG(1, LOG_LINE_NORMAL, L"         Some functionality will be impaired!");
+    }
+#endif
 } // VOID LogBasicInfo()
 
 //
@@ -446,7 +474,7 @@ efi_main (EFI_HANDLE ImageHandle, EFI_SYSTEM_TABLE *SystemTable)
     BOOLEAN            MainLoopRunning = TRUE;
     BOOLEAN            MokProtocol;
     REFIT_MENU_ENTRY   *ChosenEntry;
-    UINTN              MenuExit, i;
+    UINTN              MenuExit = MENU_EXIT_ENTER, i;
     CHAR16             *SelectionName = NULL;
     EG_PIXEL           BGColor = COLOR_LIGHTBLUE;
 
@@ -457,10 +485,10 @@ efi_main (EFI_HANDLE ImageHandle, EFI_SYSTEM_TABLE *SystemTable)
         return Status;
 
     // read configuration
-    CopyMem(GlobalConfig.ScanFor, "ieom       ", NUM_SCAN_OPTIONS);
+    MyCopyMem(GlobalConfig.ScanFor, "ieom       ", NUM_SCAN_OPTIONS);
     FindLegacyBootType();
     if (GlobalConfig.LegacyType == LEGACY_TYPE_MAC)
-       CopyMem(GlobalConfig.ScanFor, "ihebocm    ", NUM_SCAN_OPTIONS);
+       MyCopyMem(GlobalConfig.ScanFor, "ihebocm    ", NUM_SCAN_OPTIONS);
     SetConfigFilename(ImageHandle);
 
     // Scan volumes first to find SelfVolume, which is needed by LoadDrivers()
@@ -473,6 +501,7 @@ efi_main (EFI_HANDLE ImageHandle, EFI_SYSTEM_TABLE *SystemTable)
         StartLogging(FALSE);
         LogBasicInfo();
     }
+    LOG(3, LOG_LINE_NORMAL, L"GlobalConfig.DontScanFiles is '%s'", GlobalConfig.DontScanFiles);
     MokProtocol = SecureBootSetup();
     if (LoadDrivers())
         ScanVolumes();
@@ -496,6 +525,7 @@ efi_main (EFI_HANDLE ImageHandle, EFI_SYSTEM_TABLE *SystemTable)
     SetVolumeIcons();
     ScanForBootloaders(FALSE);
     ScanForTools();
+
     // SetupScreen() clears the screen; but ScanForBootloaders() may display a
     // message that must be deleted, so do so
     BltClearScreen(TRUE);
