@@ -34,7 +34,7 @@
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 /*
- * Modifications copyright (c) 2012-2021 Roderick W. Smith
+ * Modifications copyright (c) 2012-2024 Roderick W. Smith
  *
  * Modifications distributed under the terms of the GNU General Public
  * License (GPL) version 3 (GPLv3), or (at your option) any later version.
@@ -81,35 +81,27 @@
 #define MACOSX_LOADER_DIR      L"System\\Library\\CoreServices"
 #define MACOSX_LOADER_PATH      ( MACOSX_LOADER_DIR L"\\boot.efi" )
 #if defined (EFIX64)
-#define SHELL_NAMES             L"\\EFI\\tools\\shell.efi,\\EFI\\tools\\shellx64.efi,\\shell.efi,\\shellx64.efi"
-#define GPTSYNC_NAMES           L"\\EFI\\tools\\gptsync.efi,\\EFI\\tools\\gptsync_x64.efi"
-#define GDISK_NAMES             L"\\EFI\\tools\\gdisk.efi,\\EFI\\tools\\gdisk_x64.efi"
-#define NETBOOT_NAMES           L"\\EFI\\tools\\ipxe.efi"
-#define MEMTEST_NAMES           L"memtest86.efi,memtest86_x64.efi,memtest86x64.efi,bootx64.efi"
+#define GPTSYNC_NAMES           L"gptsync.efi,gptsync_x64.efi"
+#define GDISK_NAMES             L"gdisk.efi,gdisk_x64.efi"
+#define NETBOOT_NAMES           L"ipxe.efi"
 #define FALLBACK_FULLNAME       L"EFI\\BOOT\\bootx64.efi"
 #define FALLBACK_BASENAME       L"bootx64.efi"
 #elif defined (EFI32)
-#define SHELL_NAMES             L"\\EFI\\tools\\shell.efi,\\EFI\\tools\\shellia32.efi,\\shell.efi,\\shellia32.efi"
-#define GPTSYNC_NAMES           L"\\EFI\\tools\\gptsync.efi,\\EFI\\tools\\gptsync_ia32.efi"
-#define GDISK_NAMES             L"\\EFI\\tools\\gdisk.efi,\\EFI\\tools\\gdisk_ia32.efi"
-#define NETBOOT_NAMES           L"\\EFI\\tools\\ipxe.efi"
-#define MEMTEST_NAMES           L"memtest86.efi,memtest86_ia32.efi,memtest86ia32.efi,bootia32.efi"
+#define GPTSYNC_NAMES           L"gptsync.efi,gptsync_ia32.efi"
+#define GDISK_NAMES             L"gdisk.efi,gdisk_ia32.efi"
+#define NETBOOT_NAMES           L"ipxe.efi"
 #define FALLBACK_FULLNAME       L"EFI\\BOOT\\bootia32.efi"
 #define FALLBACK_BASENAME       L"bootia32.efi"
 #elif defined (EFIAARCH64)
-#define SHELL_NAMES             L"\\EFI\\tools\\shell.efi,\\EFI\\tools\\shellaa64.efi,\\shell.efi,\\shellaa64.efi"
-#define GPTSYNC_NAMES           L"\\EFI\\tools\\gptsync.efi,\\EFI\\tools\\gptsync_aa64.efi"
-#define GDISK_NAMES             L"\\EFI\\tools\\gdisk.efi,\\EFI\\tools\\gdisk_aa64.efi"
-#define NETBOOT_NAMES           L"\\EFI\\tools\\ipxe.efi"
-#define MEMTEST_NAMES           L"memtest86.efi,memtest86_aa64.efi,memtest86aa64.efi,bootaa64.efi"
+#define GPTSYNC_NAMES           L"gptsync.efi,gptsync_aa64.efi"
+#define GDISK_NAMES             L"gdisk.efi,gdisk_aa64.efi"
+#define NETBOOT_NAMES           L"ipxe.efi"
 #define FALLBACK_FULLNAME       L"EFI\\BOOT\\bootaa64.efi"
 #define FALLBACK_BASENAME       L"bootaa64.efi"
 #else
-#define SHELL_NAMES             L"\\EFI\\tools\\shell.efi,\\shell.efi"
-#define GPTSYNC_NAMES           L"\\EFI\\tools\\gptsync.efi"
-#define GDISK_NAMES             L"\\EFI\\tools\\gdisk.efi"
-#define NETBOOT_NAMES           L"\\EFI\\tools\\ipxe.efi"
-#define MEMTEST_NAMES           L"memtest86.efi"
+#define GPTSYNC_NAMES           L"gptsync.efi"
+#define GDISK_NAMES             L"gdisk.efi"
+#define NETBOOT_NAMES           L"ipxe.efi"
 #define DRIVER_DIRS             L"drivers"
 #define FALLBACK_FULLNAME       L"EFI\\BOOT\\boot.efi" /* Not really correct */
 #define FALLBACK_BASENAME       L"boot.efi"            /* Not really correct */
@@ -119,11 +111,6 @@
 #define IPXE_DISCOVER_NAME      L"\\efi\\tools\\ipxe_discover.efi"
 #define IPXE_NAME               L"\\efi\\tools\\ipxe.efi"
 
-// Patterns that identify Linux kernels. Added to the loader match pattern when the
-// scan_all_linux_kernels option is set in the configuration file. Causes kernels WITHOUT
-// a ".efi" extension to be found when scanning for boot loaders.
-#define LINUX_MATCH_PATTERNS    L"vmlinuz*,bzImage*,kernel*"
-
 EFI_GUID GlobalGuid = EFI_GLOBAL_VARIABLE;
 
 static REFIT_MENU_ENTRY MenuEntryAbout    = { L"About rEFInd", TAG_ABOUT, 1, 0, 'A', NULL, NULL, NULL };
@@ -160,7 +147,7 @@ static REFIT_MENU_SCREEN* CopyMenuScreen(REFIT_MENU_SCREEN *Entry) {
         if (Entry->TitleImage != NULL) {
             NewEntry->TitleImage = AllocatePool(sizeof(EG_IMAGE));
             if (NewEntry->TitleImage != NULL)
-                CopyMem(NewEntry->TitleImage, Entry->TitleImage, sizeof(EG_IMAGE));
+                MyCopyMem(NewEntry->TitleImage, Entry->TitleImage, sizeof(EG_IMAGE));
         } // if
         NewEntry->InfoLineCount = Entry->InfoLineCount;
         NewEntry->InfoLines = (CHAR16**) AllocateZeroPool(Entry->InfoLineCount * (sizeof(CHAR16*)));
@@ -190,17 +177,17 @@ static REFIT_MENU_ENTRY* CopyMenuEntry(REFIT_MENU_ENTRY *Entry) {
 
     NewEntry = AllocateZeroPool(sizeof(REFIT_MENU_ENTRY));
     if ((Entry != NULL) && (NewEntry != NULL)) {
-        CopyMem(NewEntry, Entry, sizeof(REFIT_MENU_ENTRY));
+        MyCopyMem(NewEntry, Entry, sizeof(REFIT_MENU_ENTRY));
         NewEntry->Title = (Entry->Title) ? StrDuplicate(Entry->Title) : NULL;
         if (Entry->BadgeImage != NULL) {
             NewEntry->BadgeImage = AllocatePool(sizeof(EG_IMAGE));
             if (NewEntry->BadgeImage != NULL)
-                CopyMem(NewEntry->BadgeImage, Entry->BadgeImage, sizeof(EG_IMAGE));
+                MyCopyMem(NewEntry->BadgeImage, Entry->BadgeImage, sizeof(EG_IMAGE));
         }
         if (Entry->Image != NULL) {
             NewEntry->Image = AllocatePool(sizeof(EG_IMAGE));
             if (NewEntry->Image != NULL)
-                CopyMem(NewEntry->Image, Entry->Image, sizeof(EG_IMAGE));
+                MyCopyMem(NewEntry->Image, Entry->Image, sizeof(EG_IMAGE));
         }
         if (Entry->SubScreen != NULL) {
             NewEntry->SubScreen = CopyMenuScreen(Entry->SubScreen);
@@ -386,7 +373,9 @@ VOID GenerateSubScreen(LOADER_ENTRY *Entry, IN REFIT_VOLUME *Volume, IN BOOLEAN
             InitrdName =  FindInitrd(Entry->LoaderPath, Volume);
             TokenCount = ReadTokenLine(File, &TokenList);
             KernelVersion = FindNumbers(Entry->LoaderPath);
-            ReplaceSubstring(&(TokenList[1]), KERNEL_VERSION, KernelVersion);
+            if (TokenCount >= 2) {
+                ReplaceSubstring(&(TokenList[1]), KERNEL_VERSION, KernelVersion);
+            }
             // first entry requires special processing, since it was initially set
             // up with a default title but correct options by InitializeSubScreen(),
             // earlier....
@@ -527,7 +516,7 @@ VOID SetLoaderDefaults(LOADER_ENTRY *Entry, CHAR16 *LoaderPath, REFIT_VOLUME *Vo
 
     LOG(4, LOG_LINE_NORMAL, L"Adding hints based on specific loaders");
     // detect specific loaders
-    if (StriSubCmp(L"bzImage", NameClues) || StriSubCmp(L"vmlinuz", NameClues) || StriSubCmp(L"kernel", NameClues)) {
+    if (IsInSubstring(NameClues, GlobalConfig.LinuxPrefixes)) {
         if (Volume->DiskKind != DISK_KIND_NET) {
             GuessLinuxDistribution(&OSIconName, Volume, LoaderPath);
             Entry->LoadOptions = GetMainLinuxOptions(LoaderPath, Volume);
@@ -581,7 +570,7 @@ VOID SetLoaderDefaults(LOADER_ENTRY *Entry, CHAR16 *LoaderPath, REFIT_VOLUME *Vo
         Entry->OSType = 'N';
         ShortcutLetter = 'N';
         MergeStrings(&OSIconName, L"network", L',');
-    } 
+    }
 
     if ((ShortcutLetter >= 'a') && (ShortcutLetter <= 'z'))
         ShortcutLetter = ShortcutLetter - 'a' + 'A'; // convert lowercase to uppercase
@@ -774,7 +763,7 @@ static BOOLEAN ShouldScan(REFIT_VOLUME *Volume, CHAR16 *Path) {
     // See if Path includes an explicit volume declaration that's NOT Volume....
     PathCopy = StrDuplicate(Path);
     if (SplitVolumeAndFilename(&PathCopy, &VolName)) {
-        if (VolName && (!MyStriCmp(VolName, Volume->FsName) ||
+        if (VolName && (!MyStriCmp(VolName, Volume->FsName) &&
             !MyStriCmp(VolName, Volume->PartName))) {
                 ScanIt = FALSE;
         } // if
@@ -893,6 +882,22 @@ static BOOLEAN IsSymbolicLink(REFIT_VOLUME *Volume, CHAR16 *FullName, EFI_FILE_I
     return (DirEntry->FileSize != FileSize2);
 } // BOOLEAN IsSymbolicLink()
 
+// Log all the current menu entries. Takes the desired log level as an option
+static VOID LogMenuEntries(UINTN LogLevel) {
+    UINTN i;
+
+    if (LogLevel <= GlobalConfig.LogLevel) {
+        LOG(LogLevel, LOG_LINE_NORMAL, L"MainMenu currently has %ld entries; they are....", MainMenu.EntryCount);
+        for (i = 0; i < MainMenu.EntryCount; i++) {
+            if (MainMenu.Entries[i]) {
+                //LOG(LogLevel, LOG_LINE_NORMAL, L"  --> MainMenu.Entries[%ld] is %lld", i, MainMenu.Entries[i]);
+                //LOG(LogLevel, LOG_LINE_NORMAL, L"  --> Menu item %ld Title address is %lld", i, MainMenu.Entries[i]->Title);
+                LOG(LogLevel, LOG_LINE_NORMAL, L"  --> Menu item %ld is '%s'", i, MainMenu.Entries[i]->Title);
+            }
+        }
+    }
+} // VOID LogMenuEntries()
+
 // Scan an individual directory for EFI boot loader files and, if found,
 // add them to the list. Exception: Ignores FALLBACK_FULLNAME, which is picked
 // up in ScanEfiFiles(). Sorts the entries within the loader directory so that
@@ -919,12 +924,14 @@ static BOOLEAN ScanLoaderDir(IN REFIT_VOLUME *Volume, IN CHAR16 *Path, IN CHAR16
           FullName = StrDuplicate(Path);
           MergeStrings(&FullName, DirEntry->FileName, L'\\');
           CleanUpPathNameSlashes(FullName);
+
+          if (!GlobalConfig.FollowSymlinks && IsSymbolicLink(Volume, FullName, DirEntry)) {
+              continue;   // is symbolic link ... skip as ignoring such
+          }
           if (DirEntry->FileName[0] == '.' ||
               MyStriCmp(Extension, L".icns") ||
               MyStriCmp(Extension, L".png") ||
               (MyStriCmp(DirEntry->FileName, FALLBACK_BASENAME) && (MyStriCmp(Path, L"EFI\\BOOT"))) ||
-              FilenameIn(Volume, Path, DirEntry->FileName, SHELL_NAMES) ||
-              IsSymbolicLink(Volume, FullName, DirEntry) || /* is symbolic link */
               HasSignedCounterpart(Volume, FullName) || /* a file with same name plus ".efi.signed" is present */
               FilenameIn(Volume, Path, DirEntry->FileName, GlobalConfig.DontScanFiles) ||
               !IsValidLoader(Volume->RootDir, FullName)) {
@@ -939,15 +946,17 @@ static BOOLEAN ScanLoaderDir(IN REFIT_VOLUME *Volume, IN CHAR16 *Path, IN CHAR16
              if (DuplicatesFallback(Volume, FullName))
                 FoundFallbackDuplicate = TRUE;
           } // if
+          if (!IsIn(Path, GlobalConfig.ExtraToolLocations)) {
+              MergeStrings(&GlobalConfig.ExtraToolLocations, Path, L',');
+              LOG(3, LOG_LINE_NORMAL, L"Adding '%s' to ExtraToolLocations", Path);
+          }
           MyFreePool(Extension);
           MyFreePool(FullName);
        } // while
 
        NewLoader = LoaderList;
        while (NewLoader != NULL) {
-           IsLinux = (StriSubCmp(L"bzImage", NewLoader->FileName) ||
-                      StriSubCmp(L"vmlinuz", NewLoader->FileName) ||
-                      StriSubCmp(L"kernel", NewLoader->FileName));
+           IsLinux = IsInSubstring(NewLoader->FileName, GlobalConfig.LinuxPrefixes);
            if ((FirstKernel != NULL) && IsLinux && GlobalConfig.FoldLinuxKernels) {
                AddKernelToSubmenu(FirstKernel, NewLoader->FileName, Volume);
            } else {
@@ -958,8 +967,9 @@ static BOOLEAN ScanLoaderDir(IN REFIT_VOLUME *Volume, IN CHAR16 *Path, IN CHAR16
            }
            NewLoader = NewLoader->NextEntry;
        } // while
-       if ((NewLoader != NULL) && (FirstKernel != NULL) && IsLinux && GlobalConfig.FoldLinuxKernels)
+       if ((FirstKernel != NULL) && IsLinux && GlobalConfig.FoldLinuxKernels)
            AddMenuEntry(FirstKernel->me.SubScreen, &MenuEntryReturn);
+       LogMenuEntries(4);
 
        CleanUpLoaderList(LoaderList);
        Status = DirIterClose(&DirIter);
@@ -1018,13 +1028,13 @@ static VOID ScanNetboot() {
             Location = RuniPXEDiscover(SelfVolume->DeviceHandle);
             if (Location != NULL && FileExists(SelfVolume->RootDir, iPXEFileName)) {
                 NetVolume = AllocatePool(sizeof(REFIT_VOLUME));
-                CopyMem(NetVolume, SelfVolume, sizeof(REFIT_VOLUME));
+                MyCopyMem(NetVolume, SelfVolume, sizeof(REFIT_VOLUME));
                 NetVolume->DiskKind = DISK_KIND_NET;
                 NetVolume->VolBadgeImage = BuiltinIcon(BUILTIN_ICON_VOL_NET);
                 NetVolume->PartName = NetVolume->VolName = NetVolume->FsName = NULL;
                 AddLoaderEntry(iPXEFileName, Location, NetVolume, TRUE);
             } // if support files exist and are valid
-    } 
+    }
 } // VOID ScanNetboot()
 
 // Adds *FullFileName as a macOS loader, if it exists.
@@ -1060,7 +1070,7 @@ static VOID ScanEfiFiles(REFIT_VOLUME *Volume) {
             Volume->PartName ? Volume->PartName : Volume->VolName);
         MatchPatterns = StrDuplicate(LOADER_MATCH_PATTERNS);
         if (GlobalConfig.ScanAllLinux)
-            MergeStrings(&MatchPatterns, LINUX_MATCH_PATTERNS, L',');
+            MergeStrings(&MatchPatterns, GlobalConfig.LinuxMatchPatterns, L',');
 
         // check for macOS boot loader
         if (ShouldScan(Volume, MACOSX_LOADER_DIR)) {
@@ -1162,7 +1172,9 @@ static VOID ScanEfiFiles(REFIT_VOLUME *Volume) {
         // for the fallback boot loader
         if (ScanFallbackLoader && FileExists(Volume->RootDir, FALLBACK_FULLNAME) && ShouldScan(Volume, L"EFI\\BOOT") &&
             !FilenameIn(Volume, L"EFI\\BOOT", FALLBACK_BASENAME, GlobalConfig.DontScanFiles)) {
-                AddLoaderEntry(FALLBACK_FULLNAME, L"Fallback boot loader", Volume, TRUE);
+                Temp = StrDuplicate(FALLBACK_FULLNAME);
+                AddLoaderEntry(Temp, L"Fallback boot loader", Volume, TRUE);
+                MyFreePool(Temp);
         }
         MyFreePool(MatchPatterns);
     } else {
@@ -1400,23 +1412,28 @@ VOID ScanForBootloaders(BOOLEAN ShowMessage) {
         MainMenu.Entries[i]->ShortcutDigit = (CHAR16)('1' + i);
 
     // wait for user ACK when there were errors
-//     SwitchToText(FALSE);
-//     FinishTextScreen(FALSE);
 } // VOID ScanForBootloaders()
 
 // Checks to see if a specified file seems to be a valid tool.
 // Returns TRUE if it passes all tests, FALSE otherwise
 static BOOLEAN IsValidTool(IN REFIT_VOLUME *BaseVolume, CHAR16 *PathName) {
-    CHAR16 *DontVolName = NULL, *DontPathName = NULL, *DontFileName = NULL, *DontScanThis;
-    CHAR16 *TestVolName = NULL, *TestPathName = NULL, *TestFileName = NULL, *DontScanTools;
+    CHAR16 *DontVolName = NULL, *DontPathName = NULL, *DontFileName = NULL, *DontScanThis = NULL;
+    CHAR16 *TestVolName = NULL, *TestPathName = NULL, *TestFileName = NULL, *DontScanTools = NULL;
     BOOLEAN retval = TRUE;
     UINTN i = 0;
 
     LOG(3, LOG_LINE_NORMAL, L"Checking validity of tool '%s' on '%s'", PathName,
         BaseVolume->PartName ? BaseVolume->PartName : BaseVolume->VolName);
-    DontScanTools = ReadHiddenTags(L"HiddenTools");
-    MergeStrings(&DontScanTools, GlobalConfig.DontScanTools, L',');
     if (FileExists(BaseVolume->RootDir, PathName) && IsValidLoader(BaseVolume->RootDir, PathName)) {
+        if (gHiddenTools == NULL) {
+            DontScanTools = ReadHiddenTags(L"HiddenTools");
+            if (DontScanTools)
+                gHiddenTools = StrDuplicate(DontScanTools);
+        } else {
+            DontScanTools = StrDuplicate(gHiddenTools);
+        }
+
+        MergeStrings(&DontScanTools, GlobalConfig.DontScanTools, L',');
         SplitPathName(PathName, &TestVolName, &TestPathName, &TestFileName);
         while (retval && (DontScanThis = FindCommaDelimited(DontScanTools, i++))) {
             SplitPathName(DontScanThis, &DontVolName, &DontPathName, &DontFileName);
@@ -1435,6 +1452,7 @@ static BOOLEAN IsValidTool(IN REFIT_VOLUME *BaseVolume, CHAR16 *PathName) {
         } // while
     } else
         retval = FALSE;
+    LOG(4, LOG_LINE_NORMAL, L"About to free multiple variables in IsValidTool()");
     MyFreePool(TestVolName);
     MyFreePool(TestPathName);
     MyFreePool(TestFileName);
@@ -1446,37 +1464,62 @@ static BOOLEAN IsValidTool(IN REFIT_VOLUME *BaseVolume, CHAR16 *PathName) {
 
 // Locate a single tool from the specified Locations using one of the
 // specified Names and add it to the menu.
-static VOID FindTool(CHAR16 *Locations, CHAR16 *Names, CHAR16 *Description, UINTN Icon) {
-    UINTN j = 0, k, VolumeIndex;
-    CHAR16 *DirName, *FileName, *PathName, *FullDescription;
+VOID FindTool(CHAR16 *Locations, CHAR16 *Names, CHAR16 *Description, UINTN Icon) {
+    UINTN j, k, VolumeIndex;
+    CHAR16 *DirName, *FileName, *PathName, *FullDescription, *VolName = NULL;
+    CHAR16 *ScannedLocations = NULL;
+    BOOLEAN VolMatch;
 
-    while ((DirName = FindCommaDelimited(Locations, j++)) != NULL) {
-        k = 0;
-        while ((FileName = FindCommaDelimited(Names, k++)) != NULL) {
-            PathName = StrDuplicate(DirName);
-            MergeStrings(&PathName, FileName, MyStriCmp(PathName, L"\\") ? 0 : L'\\');
-            for (VolumeIndex = 0; VolumeIndex < VolumesCount; VolumeIndex++) {
-                if ((Volumes[VolumeIndex]->RootDir != NULL) && (IsValidTool(Volumes[VolumeIndex], PathName))) {
-                    FullDescription = PoolPrint(L"%s at %s on %s", Description, PathName,
-                                                Volumes[VolumeIndex]->VolName);
-                    LOG(1, LOG_LINE_NORMAL, L"Adding tag for '%s' on '%s'", FileName,
-                        Volumes[VolumeIndex]->VolName);
-                    AddToolEntry(Volumes[VolumeIndex], PathName, FullDescription,
-                                 BuiltinIcon(Icon), 'S', FALSE);
-                    MyFreePool(FullDescription);
-                } // if
-            } // for
-            MyFreePool(PathName);
-            MyFreePool(FileName);
-        } // while Names
-        MyFreePool(DirName);
-    } // while Locations
+    LOG(1, LOG_LINE_NORMAL, L"Scanning for tools '%s' in '%s'", Names, Locations);
+    for (VolumeIndex = 0; VolumeIndex < VolumesCount; VolumeIndex++) {
+        LOG(2, LOG_LINE_NORMAL, L"FindTool: Scanning volume '%s'", Volumes[VolumeIndex]->VolName);
+        if (Volumes[VolumeIndex]->RootDir != NULL) {
+            j = 0;
+            while ((DirName = FindCommaDelimited(Locations, j++)) != NULL) {
+                if (!IsIn(DirName, ScannedLocations)) {
+                    MergeStrings(&ScannedLocations, DirName, ',');
+                    SplitVolumeAndFilename(&DirName, &VolName);
+                    VolMatch = ((VolName == NULL) ||
+                                (MyStriCmp(VolName, Volumes[VolumeIndex]->FsName)) ||
+                                (MyStriCmp(VolName, Volumes[VolumeIndex]->PartName)) ||
+                                (MyStriCmp(VolName, Volumes[VolumeIndex]->VolName)));
+                    if (VolMatch && (FileExists(Volumes[VolumeIndex]->RootDir, DirName))) {
+                        k = 0;
+                        while ((FileName = FindCommaDelimited(Names, k++)) != NULL) {
+                            PathName = StrDuplicate(DirName);
+                            LOG(3, LOG_LINE_NORMAL, L"FindTool: Checking volume '%s' for '%s'",
+                                Volumes[VolumeIndex]->VolName, PathName);
+                            MergeStrings(&PathName, FileName, MyStriCmp(PathName, L"\\") ? 0 : L'\\');
+                            if (IsValidTool(Volumes[VolumeIndex], PathName)) {
+                                FullDescription = PoolPrint(L"%s at %s on %s", Description, PathName,
+                                                            Volumes[VolumeIndex]->VolName);
+                                LOG(1, LOG_LINE_NORMAL, L"Adding tag for '%s' on '%s'", FileName,
+                                    Volumes[VolumeIndex]->VolName);
+                                AddToolEntry(Volumes[VolumeIndex], PathName, FullDescription,
+                                             BuiltinIcon(Icon), 'S', FALSE);
+                                MyFreePool(FullDescription);
+                            MyFreePool(PathName);
+                            MyFreePool(FileName);
+                            } // if
+                        } // while Names
+                    } // if VolMatch....
+                    MyFreePool(VolName);
+                    VolName = NULL;
+                } else {
+                    LOG(3, LOG_LINE_NORMAL, L"Skipping duplicate directory name: '%s'", DirName);
+                }
+                MyFreePool(DirName);
+            } // while Locations
+            MyFreePool(ScannedLocations);
+            ScannedLocations = NULL;
+        } // if RootDir != NULL
+    } // for
 } // VOID FindTool()
 
 // Add the second-row tags containing built-in and external tools (EFI shell,
 // reboot, etc.)
 VOID ScanForTools(VOID) {
-    CHAR16 *FileName = NULL, *VolName = NULL, *MokLocations, *Description;
+    CHAR16 *FileName = NULL, *VolName = NULL, *Description, *AllToolLocations;
     REFIT_MENU_ENTRY *TempMenuEntry;
     UINTN i, j, VolumeIndex;
     UINT64 osind;
@@ -1484,38 +1527,35 @@ VOID ScanForTools(VOID) {
     UINT32 CsrValue;
 
     LOG(1, LOG_LINE_SEPARATOR, L"Scanning for tools");
-    MokLocations = StrDuplicate(MOK_LOCATIONS);
-    if (MokLocations != NULL)
-        MergeStrings(&MokLocations, SelfDirPath, L',');
+    if (!IsIn(GlobalConfig.ExtraToolLocations, SelfDirPath))
+        MergeStrings(&GlobalConfig.ExtraToolLocations, SelfDirPath, L',');
 
+    AllToolLocations = StrDuplicate(GlobalConfig.ToolLocations);
+    MergeStrings(&AllToolLocations, GlobalConfig.ExtraToolLocations, L',');
     for (i = 0; i < NUM_TOOLS; i++) {
         switch(GlobalConfig.ShowTools[i]) {
             // NOTE: Be sure that FileName is NULL at the end of each case.
             case TAG_SHUTDOWN:
                 TempMenuEntry = CopyMenuEntry(&MenuEntryShutdown);
                 TempMenuEntry->Image = BuiltinIcon(BUILTIN_ICON_FUNC_SHUTDOWN);
-                LOG(2, LOG_LINE_NORMAL, L"Adding Shutdown tag");
                 AddMenuEntry(&MainMenu, TempMenuEntry);
                 break;
 
             case TAG_REBOOT:
                 TempMenuEntry = CopyMenuEntry(&MenuEntryReset);
                 TempMenuEntry->Image = BuiltinIcon(BUILTIN_ICON_FUNC_RESET);
-                LOG(2, LOG_LINE_NORMAL, L"Adding Reboot tag");
                 AddMenuEntry(&MainMenu, TempMenuEntry);
                 break;
 
             case TAG_ABOUT:
                 TempMenuEntry = CopyMenuEntry(&MenuEntryAbout);
                 TempMenuEntry->Image = BuiltinIcon(BUILTIN_ICON_FUNC_ABOUT);
-                LOG(2, LOG_LINE_NORMAL, L"Adding Info/About tag");
                 AddMenuEntry(&MainMenu, TempMenuEntry);
                 break;
 
             case TAG_EXIT:
                 TempMenuEntry = CopyMenuEntry(&MenuEntryExit);
                 TempMenuEntry->Image = BuiltinIcon(BUILTIN_ICON_FUNC_EXIT);
-                LOG(2, LOG_LINE_NORMAL, L"Adding Exit tag");
                 AddMenuEntry(&MainMenu, TempMenuEntry);
                 break;
 
@@ -1523,7 +1563,6 @@ VOID ScanForTools(VOID) {
                 if (GlobalConfig.HiddenTags) {
                     TempMenuEntry = CopyMenuEntry(&MenuEntryManageHidden);
                     TempMenuEntry->Image = BuiltinIcon(BUILTIN_ICON_FUNC_HIDDEN);
-                    LOG(2, LOG_LINE_NORMAL, L"Adding Hidden tag");
                     AddMenuEntry(&MainMenu, TempMenuEntry);
                 }
                 break;
@@ -1534,7 +1573,6 @@ VOID ScanForTools(VOID) {
                     if (osind & EFI_OS_INDICATIONS_BOOT_TO_FW_UI) {
                         TempMenuEntry = CopyMenuEntry(&MenuEntryFirmware);
                         TempMenuEntry->Image = BuiltinIcon(BUILTIN_ICON_FUNC_FIRMWARE);
-                        LOG(2, LOG_LINE_NORMAL, L"Adding Reboot-to-Firmware tag");
                         AddMenuEntry(&MainMenu, TempMenuEntry);
                     } else {
                         LOG(1, LOG_LINE_NORMAL, L"showtools includes firmware, but EFI lacks support");
@@ -1546,62 +1584,32 @@ VOID ScanForTools(VOID) {
                 break;
 
             case TAG_SHELL:
-                j = 0;
-                while ((FileName = FindCommaDelimited(SHELL_NAMES, j++)) != NULL) {
-                    if (IsValidTool(SelfVolume, FileName)) {
-                        LOG(1, LOG_LINE_NORMAL, L"Adding Shell tag for '%s' on '%s'", FileName,
-                            SelfVolume->VolName);
-                        AddToolEntry(SelfVolume, FileName, L"EFI Shell",
-                                     BuiltinIcon(BUILTIN_ICON_TOOL_SHELL),
-                                     'S', FALSE);
-                    }
-                    MyFreePool(FileName);
-                    FileName = NULL;
-                } // while
+                FindTool(AllToolLocations, SHELL_NAMES, L"EFI shell", BUILTIN_ICON_TOOL_SHELL);
                 ScanFirmwareDefined(1, L"Shell", BuiltinIcon(BUILTIN_ICON_TOOL_SHELL));
                 break;
 
             case TAG_GPTSYNC:
-                j = 0;
-                while ((FileName = FindCommaDelimited(GPTSYNC_NAMES, j++)) != NULL) {
-                    if (IsValidTool(SelfVolume, FileName)) {
-                        LOG(1, LOG_LINE_NORMAL, L"Adding Hybrid MBR tool tag for '%s' on '%s'", FileName,
-                            SelfVolume->VolName);
-                        AddToolEntry(SelfVolume, FileName, L"Hybrid MBR tool",
-                                     BuiltinIcon(BUILTIN_ICON_TOOL_PART),
-                                     'P', FALSE);
-                    } // if
-                    MyFreePool(FileName);
-                } // while
-                FileName = NULL;
+                FindTool(AllToolLocations, GPTSYNC_NAMES, L"Hybrid MBR tool", BUILTIN_ICON_TOOL_PART);
                 break;
 
             case TAG_GDISK:
-                j = 0;
-                while ((FileName = FindCommaDelimited(GDISK_NAMES, j++)) != NULL) {
-                    if (IsValidTool(SelfVolume, FileName)) {
-                        LOG(1, LOG_LINE_NORMAL, L"Adding GPT fdisk tag for '%s' on '%s'", FileName,
-                            SelfVolume->VolName);
-                        AddToolEntry(SelfVolume, FileName, L"disk partitioning tool",
-                                     BuiltinIcon(BUILTIN_ICON_TOOL_PART), 'G', FALSE);
-                    } // if
-                    MyFreePool(FileName);
-                } // while
-                FileName = NULL;
+                FindTool(AllToolLocations, GDISK_NAMES, L"Disk partitioning tool", BUILTIN_ICON_TOOL_PART);
                 break;
-            
+
             case TAG_NETBOOT:
-                j = 0;
-                while ((FileName = FindCommaDelimited(NETBOOT_NAMES, j++)) != NULL) {
-                    if (IsValidTool(SelfVolume, FileName)) {
-                        LOG(1, LOG_LINE_NORMAL, L"Adding Netboot tag for '%s' on '%s'", FileName,
-                            SelfVolume->VolName);
-                        AddToolEntry(SelfVolume, FileName, L"Netboot",
-                                     BuiltinIcon(BUILTIN_ICON_TOOL_NETBOOT), 'N', FALSE);
-                    } // if
-                    MyFreePool(FileName);
-                } // while
-                FileName = NULL;
+                FindTool(AllToolLocations, NETBOOT_NAMES, L"Netboot", BUILTIN_ICON_TOOL_NETBOOT);
+                break;
+
+            case TAG_MOK_TOOL:
+                FindTool(AllToolLocations, MOK_NAMES, L"MOK utility", BUILTIN_ICON_TOOL_MOK_TOOL);
+                break;
+
+            case TAG_FWUPDATE_TOOL:
+                FindTool(AllToolLocations, FWUPDATE_NAMES, L"firmware update utility", BUILTIN_ICON_TOOL_FWUPDATE);
+                break;
+
+            case TAG_MEMTEST:
+                FindTool(AllToolLocations, MEMTEST_NAMES, L"Memory test utility", BUILTIN_ICON_TOOL_MEMTEST);
                 break;
 
             case TAG_APPLE_RECOVERY:
@@ -1646,19 +1654,10 @@ VOID ScanForTools(VOID) {
                 FileName = NULL;
                 break;
 
-            case TAG_MOK_TOOL:
-                FindTool(MokLocations, MOK_NAMES, L"MOK utility", BUILTIN_ICON_TOOL_MOK_TOOL);
-                break;
-
-            case TAG_FWUPDATE_TOOL:
-                FindTool(MokLocations, FWUPDATE_NAMES, L"firmware update utility", BUILTIN_ICON_TOOL_FWUPDATE);
-                break;
-
             case TAG_CSR_ROTATE:
                 if ((GetCsrStatus(&CsrValue) == EFI_SUCCESS) && (GlobalConfig.CsrValues)) {
                     TempMenuEntry = CopyMenuEntry(&MenuEntryRotateCsr);
                     TempMenuEntry->Image = BuiltinIcon(BUILTIN_ICON_FUNC_CSR_ROTATE);
-                    LOG(1, LOG_LINE_NORMAL, L"Adding CSR Rotate tag");
                     AddMenuEntry(&MainMenu, TempMenuEntry);
                 } // if
                 break;
@@ -1666,22 +1665,17 @@ VOID ScanForTools(VOID) {
             case TAG_INSTALL:
                 TempMenuEntry = CopyMenuEntry(&MenuEntryInstall);
                 TempMenuEntry->Image = BuiltinIcon(BUILTIN_ICON_FUNC_INSTALL);
-                LOG(1, LOG_LINE_NORMAL, L"Adding Install tag");
                 AddMenuEntry(&MainMenu, TempMenuEntry);
                 break;
 
             case TAG_BOOTORDER:
                 TempMenuEntry = CopyMenuEntry(&MenuEntryBootorder);
                 TempMenuEntry->Image = BuiltinIcon(BUILTIN_ICON_FUNC_BOOTORDER);
-                LOG(1, LOG_LINE_NORMAL, L"Adding Boot Order tag");
                 AddMenuEntry(&MainMenu, TempMenuEntry);
                 break;
 
-            case TAG_MEMTEST:
-                FindTool(MEMTEST_LOCATIONS, MEMTEST_NAMES, L"Memory test utility", BUILTIN_ICON_TOOL_MEMTEST);
-                break;
-
         } // switch()
     } // for
+    MyFreePool(AllToolLocations);
     LOG(2, LOG_LINE_NORMAL, L"Done scanning for tools");
 } // VOID ScanForTools
